# How to become a contributor and submit your own code

## Contributor License Agreements

We'd love to accept your sample apps and patches! Before we can take them, we
have to jump a couple of legal hurdles.

Please fill out either the individual or corporate Contributor License
Agreement (CLA).

  * If you are an individual writing original source code and you're sure you
    own the intellectual property, then you'll need to sign an [individual CLA]
    (https://developers.google.com/open-source/cla/individual).
  * If you work for a company that wants to allow you to contribute your work,
    then you'll need to sign a [corporate CLA]
    (https://developers.google.com/open-source/cla/corporate).

Follow either of the two links above to access the appropriate CLA and
instructions for how to sign and return it. Once we receive it, we'll
be able to accept your pull requests.

## Contributing A Patch

1. Submit an issue describing your proposed change to the repo in question.
1. The repo owner will respond to your issue promptly.
1. If your proposed change is accepted, and you haven't already done so, sign a
   Contributor License Agreement (see details above).
1. Fork the desired repo, develop and test your code changes.
1. Ensure that your code adheres to the existing style in the sample to which
   you are contributing.
1. Ensure that your code has an appropriate set of unit tests which all pass.
1. Submit a pull request.

## Style

We adhere as much as possible to the [ruby-style-guide][] and make the code
[rubocop][] compliant. Tests are setup to fail if there are style guide
violations.

## Testing

Please make sure all tests pass before sending a patch. This will help us to
approve your change faster.

As a matter of policy the master branch is always passing all tests, and changes
that break tests cannot be accepted. If that's your case reach out and we can
help you get it fixed.

### Running Tests

```
gem install bundler
bundle install
bundle exec rspec
bundle exec rubocop
```

## Auto generated files

Various of the files in this repository are automatically generated by
chef-codegen. Such files contain a prominent comment warning for its
auto generated origins. However some types, such as JSON or MD, do not allow
embedding comments without breaking the file or causing side effects.

### Changing auto generated files

Of course these files are not perfect there will inevitably be issues with them.
If you find an issue with them there are 2 options:

1. Send a patch to the affected files to us and we'll update the source used to
   generate the file, thus addressing the issue. Note that in this option your
   patch will not be accepted but will be used as a guide to fix the original
   file.

2. Change the file directly in the sources used by chef-codegen. By taking
   this approach your change will be attributed to you, as you'd be the author
   of the change. If you'd like to take credit for the change this is the
   recommended approach. This approach has the nice side effect to fix all other
   projects that have the same issue at once.

### File list

The list below contains all the files that were automatically generated by
chef-codegen:

  * .gitignore
  * .rubocop.yml
  * Berksfile
  * CHANGELOG.md
  * chefignore
  * CONTRIBUTING.md
  * Gemfile
  * libraries/__init__.rb
  * libraries/google/compute/network/base.rb
  * libraries/google/compute/network/delete.rb
  * libraries/google/compute/network/get.rb
  * libraries/google/compute/network/post.rb
  * libraries/google/compute/network/put.rb
  * libraries/google/compute/property/address_address.rb
  * libraries/google/compute/property/address_address_type.rb
  * libraries/google/compute/property/array.rb
  * libraries/google/compute/property/backendservice_backends.rb
  * libraries/google/compute/property/backendservice_cache_key_policy.rb
  * libraries/google/compute/property/backendservice_cdn_policy.rb
  * libraries/google/compute/property/backendservice_connection_draining.rb
  * libraries/google/compute/property/backendservice_iap.rb
  * libraries/google/compute/property/backendservice_selflink.rb
  * libraries/google/compute/property/boolean.rb
  * libraries/google/compute/property/disk_disk_encryption_key.rb
  * libraries/google/compute/property/disk_name.rb
  * libraries/google/compute/property/disk_selflink.rb
  * libraries/google/compute/property/disk_source_image_encryption_key.rb
  * libraries/google/compute/property/disk_source_snapshot_encryption_key.rb
  * libraries/google/compute/property/disktype_deprecated.rb
  * libraries/google/compute/property/disktype_selflink.rb
  * libraries/google/compute/property/double.rb
  * libraries/google/compute/property/enum.rb
  * libraries/google/compute/property/firewall_allowed.rb
  * libraries/google/compute/property/firewall_denied.rb
  * libraries/google/compute/property/forwardingrule_selflink.rb
  * libraries/google/compute/property/health_check_proxy_header.rb
  * libraries/google/compute/property/healthcheck_http_health_check.rb
  * libraries/google/compute/property/healthcheck_https_health_check.rb
  * libraries/google/compute/property/healthcheck_ssl_health_check.rb
  * libraries/google/compute/property/healthcheck_tcp_health_check.rb
  * libraries/google/compute/property/httphealthcheck_selflink.rb
  * libraries/google/compute/property/image_deprecated.rb
  * libraries/google/compute/property/image_guest_os_features.rb
  * libraries/google/compute/property/image_image_encryption_key.rb
  * libraries/google/compute/property/image_raw_disk.rb
  * libraries/google/compute/property/image_source_disk_encryption_key.rb
  * libraries/google/compute/property/instance_access_configs.rb
  * libraries/google/compute/property/instance_alias_ip_ranges.rb
  * libraries/google/compute/property/instance_disk_encryption_key.rb
  * libraries/google/compute/property/instance_disks.rb
  * libraries/google/compute/property/instance_guest_accelerators.rb
  * libraries/google/compute/property/instance_initialize_params.rb
  * libraries/google/compute/property/instance_network_interfaces.rb
  * libraries/google/compute/property/instance_scheduling.rb
  * libraries/google/compute/property/instance_selflink.rb
  * libraries/google/compute/property/instance_service_accounts.rb
  * libraries/google/compute/property/instance_source_image_encryption_key.rb
  * libraries/google/compute/property/instance_tags.rb
  * libraries/google/compute/property/instancegroup_named_ports.rb
  * libraries/google/compute/property/instancegroup_selflink.rb
  * libraries/google/compute/property/instancegroupmanager_current_actions.rb
  * libraries/google/compute/property/instancegroupmanager_named_ports.rb
  * libraries/google/compute/property/instancetemplate_access_configs.rb
  * libraries/google/compute/property/instancetemplate_alias_ip_ranges.rb
  * libraries/google/compute/property/instancetemplate_disk_encryption_key.rb
  * libraries/google/compute/property/instancetemplate_disks.rb
  * libraries/google/compute/property/instancetemplate_guest_accelerators.rb
  * libraries/google/compute/property/instancetemplate_initialize_params.rb
  * libraries/google/compute/property/instancetemplate_network_interfaces.rb
  * libraries/google/compute/property/instancetemplate_properties.rb
  * libraries/google/compute/property/instancetemplate_scheduling.rb
  * libraries/google/compute/property/instancetemplate_selflink.rb
  * libraries/google/compute/property/instancetemplate_service_accounts.rb
  * libraries/google/compute/property/instancetemplate_source_image_encryption_key.rb
  * libraries/google/compute/property/instancetemplate_tags.rb
  * libraries/google/compute/property/integer.rb
  * libraries/google/compute/property/license_selflink.rb
  * libraries/google/compute/property/machinetype_deprecated.rb
  * libraries/google/compute/property/machinetype_name.rb
  * libraries/google/compute/property/machinetype_selflink.rb
  * libraries/google/compute/property/namevalues.rb
  * libraries/google/compute/property/network_selflink.rb
  * libraries/google/compute/property/region_deprecated.rb
  * libraries/google/compute/property/region_name.rb
  * libraries/google/compute/property/region_selflink.rb
  * libraries/google/compute/property/router_advertise_mode.rb
  * libraries/google/compute/property/router_advertised_ip_ranges.rb
  * libraries/google/compute/property/router_bgp.rb
  * libraries/google/compute/property/snapshot_selflink.rb
  * libraries/google/compute/property/snapshot_snapshot_encryption_key.rb
  * libraries/google/compute/property/snapshot_source_disk_encryption_key.rb
  * libraries/google/compute/property/sslcertificate_selflink.rb
  * libraries/google/compute/property/string.rb
  * libraries/google/compute/property/string_array.rb
  * libraries/google/compute/property/subnetwork_selflink.rb
  * libraries/google/compute/property/targetpool_selflink.rb
  * libraries/google/compute/property/targetvpngateway_selflink.rb
  * libraries/google/compute/property/time.rb
  * libraries/google/compute/property/urlmap_host_rules.rb
  * libraries/google/compute/property/urlmap_path_matchers.rb
  * libraries/google/compute/property/urlmap_path_rules.rb
  * libraries/google/compute/property/urlmap_selflink.rb
  * libraries/google/compute/property/urlmap_tests.rb
  * libraries/google/compute/property/zone_deprecated.rb
  * libraries/google/compute/property/zone_name.rb
  * libraries/google/functions/gcompute_address_ip.rb
  * libraries/google/functions/gcompute_health_check_ref.rb
  * libraries/google/functions/gcompute_image_family.rb
  * metadata.rb
  * README.md
  * recipes/examples~address.rb
  * recipes/examples~backend_bucket.rb
  * recipes/examples~backend_service.rb
  * recipes/examples~delete_address.rb
  * recipes/examples~delete_backend_bucket.rb
  * recipes/examples~delete_backend_service.rb
  * recipes/examples~delete_disk.rb
  * recipes/examples~delete_firewall.rb
  * recipes/examples~delete_forwarding_rule.rb
  * recipes/examples~delete_global_address.rb
  * recipes/examples~delete_global_forwarding_rule.rb
  * recipes/examples~delete_health_check.rb
  * recipes/examples~delete_http_health_check.rb
  * recipes/examples~delete_https_health_check.rb
  * recipes/examples~delete_image.rb
  * recipes/examples~delete_instance.rb
  * recipes/examples~delete_instance_group.rb
  * recipes/examples~delete_instance_group_manager.rb
  * recipes/examples~delete_instance_template.rb
  * recipes/examples~delete_network.rb
  * recipes/examples~delete_route.rb
  * recipes/examples~delete_snapshot.rb
  * recipes/examples~delete_ssl_certificate.rb
  * recipes/examples~delete_subnetwork.rb
  * recipes/examples~delete_target_http_proxy.rb
  * recipes/examples~delete_target_https_proxy.rb
  * recipes/examples~delete_target_ssl_proxy.rb
  * recipes/examples~delete_target_tcp_proxy.rb
  * recipes/examples~delete_url_map.rb
  * recipes/examples~disk.rb
  * recipes/examples~disk_type.rb
  * recipes/examples~firewall.rb
  * recipes/examples~firewall~change1.rb
  * recipes/examples~forwarding_rule.rb
  * recipes/examples~global_address.rb
  * recipes/examples~global_forwarding_rule.rb
  * recipes/examples~health_check.rb
  * recipes/examples~http_health_check.rb
  * recipes/examples~https_health_check.rb
  * recipes/examples~image.rb
  * recipes/examples~instance.rb
  * recipes/examples~instance_group.rb
  * recipes/examples~instance_group_manager.rb
  * recipes/examples~instance_template.rb
  * recipes/examples~license.rb
  * recipes/examples~machine_type.rb
  * recipes/examples~network~auto.rb
  * recipes/examples~network~convert_to_custom.rb
  * recipes/examples~network~custom.rb
  * recipes/examples~network~legacy.rb
  * recipes/examples~region.rb
  * recipes/examples~route.rb
  * recipes/examples~snapshot.rb
  * recipes/examples~ssl_certificate.rb
  * recipes/examples~subnetwork.rb
  * recipes/examples~target_http_proxy.rb
  * recipes/examples~target_https_proxy.rb
  * recipes/examples~target_ssl_proxy.rb
  * recipes/examples~target_tcp_proxy.rb
  * recipes/examples~target_vpn_gateway.rb
  * recipes/examples~url_map.rb
  * recipes/examples~vpn_tunnel.rb
  * recipes/examples~zone.rb
  * recipes/README.md
  * recipes/tests~address.rb
  * recipes/tests~backend_bucket.rb
  * recipes/tests~backend_service.rb
  * recipes/tests~delete_address.rb
  * recipes/tests~delete_backend_bucket.rb
  * recipes/tests~delete_backend_service.rb
  * recipes/tests~delete_disk.rb
  * recipes/tests~delete_firewall.rb
  * recipes/tests~delete_forwarding_rule.rb
  * recipes/tests~delete_global_address.rb
  * recipes/tests~delete_global_forwarding_rule.rb
  * recipes/tests~delete_health_check.rb
  * recipes/tests~delete_http_health_check.rb
  * recipes/tests~delete_https_health_check.rb
  * recipes/tests~delete_image.rb
  * recipes/tests~delete_instance.rb
  * recipes/tests~delete_instance_group.rb
  * recipes/tests~delete_instance_group_manager.rb
  * recipes/tests~delete_instance_template.rb
  * recipes/tests~delete_network.rb
  * recipes/tests~delete_route.rb
  * recipes/tests~delete_snapshot.rb
  * recipes/tests~delete_ssl_certificate.rb
  * recipes/tests~delete_subnetwork.rb
  * recipes/tests~delete_target_http_proxy.rb
  * recipes/tests~delete_target_https_proxy.rb
  * recipes/tests~delete_target_ssl_proxy.rb
  * recipes/tests~delete_target_tcp_proxy.rb
  * recipes/tests~delete_url_map.rb
  * recipes/tests~disk.rb
  * recipes/tests~disk_type.rb
  * recipes/tests~firewall.rb
  * recipes/tests~firewall~change1.rb
  * recipes/tests~forwarding_rule.rb
  * recipes/tests~global_address.rb
  * recipes/tests~global_forwarding_rule.rb
  * recipes/tests~health_check.rb
  * recipes/tests~http_health_check.rb
  * recipes/tests~https_health_check.rb
  * recipes/tests~image.rb
  * recipes/tests~instance.rb
  * recipes/tests~instance_group.rb
  * recipes/tests~instance_group_manager.rb
  * recipes/tests~instance_template.rb
  * recipes/tests~license.rb
  * recipes/tests~machine_type.rb
  * recipes/tests~network~auto.rb
  * recipes/tests~network~convert_to_custom.rb
  * recipes/tests~network~custom.rb
  * recipes/tests~network~legacy.rb
  * recipes/tests~region.rb
  * recipes/tests~route.rb
  * recipes/tests~snapshot.rb
  * recipes/tests~ssl_certificate.rb
  * recipes/tests~subnetwork.rb
  * recipes/tests~target_http_proxy.rb
  * recipes/tests~target_https_proxy.rb
  * recipes/tests~target_ssl_proxy.rb
  * recipes/tests~target_tcp_proxy.rb
  * recipes/tests~target_vpn_gateway.rb
  * recipes/tests~url_map.rb
  * recipes/tests~vpn_tunnel.rb
  * recipes/tests~zone.rb
  * resources/address.rb
  * resources/backend_bucket.rb
  * resources/backend_service.rb
  * resources/disk.rb
  * resources/disk_type.rb
  * resources/firewall.rb
  * resources/forwarding_rule.rb
  * resources/global_address.rb
  * resources/global_forwarding_rule.rb
  * resources/health_check.rb
  * resources/http_health_check.rb
  * resources/https_health_check.rb
  * resources/image.rb
  * resources/instance.rb
  * resources/instance_group.rb
  * resources/instance_group_manager.rb
  * resources/instance_template.rb
  * resources/license.rb
  * resources/machine_type.rb
  * resources/network.rb
  * resources/region.rb
  * resources/route.rb
  * resources/router.rb
  * resources/snapshot.rb
  * resources/ssl_certificate.rb
  * resources/subnetwork.rb
  * resources/target_http_proxy.rb
  * resources/target_https_proxy.rb
  * resources/target_pool.rb
  * resources/target_ssl_proxy.rb
  * resources/target_tcp_proxy.rb
  * resources/target_vpn_gateway.rb
  * resources/url_map.rb
  * resources/vpn_tunnel.rb
  * resources/zone.rb
  * spec/address_spec.rb
  * spec/backend_bucket_spec.rb
  * spec/backend_service_spec.rb
  * spec/bundle.rb
  * spec/cookbooks/google-gauth/metadata.rb
  * spec/data/network/gcompute_address/success1~name.yaml
  * spec/data/network/gcompute_address/success1~title.yaml
  * spec/data/network/gcompute_address/success2~name.yaml
  * spec/data/network/gcompute_address/success2~title.yaml
  * spec/data/network/gcompute_address/success3~name.yaml
  * spec/data/network/gcompute_address/success3~title.yaml
  * spec/data/network/gcompute_backend_bucket/success1~name.yaml
  * spec/data/network/gcompute_backend_bucket/success1~title.yaml
  * spec/data/network/gcompute_backend_bucket/success2~name.yaml
  * spec/data/network/gcompute_backend_bucket/success2~title.yaml
  * spec/data/network/gcompute_backend_bucket/success3~name.yaml
  * spec/data/network/gcompute_backend_bucket/success3~title.yaml
  * spec/data/network/gcompute_backend_service/success1~name.yaml
  * spec/data/network/gcompute_backend_service/success1~title.yaml
  * spec/data/network/gcompute_backend_service/success2~name.yaml
  * spec/data/network/gcompute_backend_service/success2~title.yaml
  * spec/data/network/gcompute_backend_service/success3~name.yaml
  * spec/data/network/gcompute_backend_service/success3~title.yaml
  * spec/data/network/gcompute_disk/success1~name.yaml
  * spec/data/network/gcompute_disk/success1~title.yaml
  * spec/data/network/gcompute_disk/success2~name.yaml
  * spec/data/network/gcompute_disk/success2~title.yaml
  * spec/data/network/gcompute_disk/success3~name.yaml
  * spec/data/network/gcompute_disk/success3~title.yaml
  * spec/data/network/gcompute_disk_type/success1~name.yaml
  * spec/data/network/gcompute_disk_type/success1~title.yaml
  * spec/data/network/gcompute_disk_type/success2~name.yaml
  * spec/data/network/gcompute_disk_type/success2~title.yaml
  * spec/data/network/gcompute_disk_type/success3~name.yaml
  * spec/data/network/gcompute_disk_type/success3~title.yaml
  * spec/data/network/gcompute_firewall/success1~name.yaml
  * spec/data/network/gcompute_firewall/success1~title.yaml
  * spec/data/network/gcompute_firewall/success2~name.yaml
  * spec/data/network/gcompute_firewall/success2~title.yaml
  * spec/data/network/gcompute_firewall/success3~name.yaml
  * spec/data/network/gcompute_firewall/success3~title.yaml
  * spec/data/network/gcompute_forwarding_rule/success1~name.yaml
  * spec/data/network/gcompute_forwarding_rule/success1~title.yaml
  * spec/data/network/gcompute_forwarding_rule/success2~name.yaml
  * spec/data/network/gcompute_forwarding_rule/success2~title.yaml
  * spec/data/network/gcompute_forwarding_rule/success3~name.yaml
  * spec/data/network/gcompute_forwarding_rule/success3~title.yaml
  * spec/data/network/gcompute_global_address/success1~name.yaml
  * spec/data/network/gcompute_global_address/success1~title.yaml
  * spec/data/network/gcompute_global_address/success2~name.yaml
  * spec/data/network/gcompute_global_address/success2~title.yaml
  * spec/data/network/gcompute_global_address/success3~name.yaml
  * spec/data/network/gcompute_global_address/success3~title.yaml
  * spec/data/network/gcompute_global_forwarding_rule/success1~name.yaml
  * spec/data/network/gcompute_global_forwarding_rule/success1~title.yaml
  * spec/data/network/gcompute_global_forwarding_rule/success2~name.yaml
  * spec/data/network/gcompute_global_forwarding_rule/success2~title.yaml
  * spec/data/network/gcompute_global_forwarding_rule/success3~name.yaml
  * spec/data/network/gcompute_global_forwarding_rule/success3~title.yaml
  * spec/data/network/gcompute_health_check/success1~name.yaml
  * spec/data/network/gcompute_health_check/success1~title.yaml
  * spec/data/network/gcompute_health_check/success2~name.yaml
  * spec/data/network/gcompute_health_check/success2~title.yaml
  * spec/data/network/gcompute_health_check/success3~name.yaml
  * spec/data/network/gcompute_health_check/success3~title.yaml
  * spec/data/network/gcompute_http_health_check/success1~name.yaml
  * spec/data/network/gcompute_http_health_check/success1~title.yaml
  * spec/data/network/gcompute_http_health_check/success2~name.yaml
  * spec/data/network/gcompute_http_health_check/success2~title.yaml
  * spec/data/network/gcompute_http_health_check/success3~name.yaml
  * spec/data/network/gcompute_http_health_check/success3~title.yaml
  * spec/data/network/gcompute_https_health_check/success1~name.yaml
  * spec/data/network/gcompute_https_health_check/success1~title.yaml
  * spec/data/network/gcompute_https_health_check/success2~name.yaml
  * spec/data/network/gcompute_https_health_check/success2~title.yaml
  * spec/data/network/gcompute_https_health_check/success3~name.yaml
  * spec/data/network/gcompute_https_health_check/success3~title.yaml
  * spec/data/network/gcompute_image/success1~name.yaml
  * spec/data/network/gcompute_image/success1~title.yaml
  * spec/data/network/gcompute_image/success2~name.yaml
  * spec/data/network/gcompute_image/success2~title.yaml
  * spec/data/network/gcompute_image/success3~name.yaml
  * spec/data/network/gcompute_image/success3~title.yaml
  * spec/data/network/gcompute_instance/success1~name.yaml
  * spec/data/network/gcompute_instance/success1~title.yaml
  * spec/data/network/gcompute_instance/success2~name.yaml
  * spec/data/network/gcompute_instance/success2~title.yaml
  * spec/data/network/gcompute_instance/success3~name.yaml
  * spec/data/network/gcompute_instance/success3~title.yaml
  * spec/data/network/gcompute_instance_group/success1~name.yaml
  * spec/data/network/gcompute_instance_group/success1~title.yaml
  * spec/data/network/gcompute_instance_group/success2~name.yaml
  * spec/data/network/gcompute_instance_group/success2~title.yaml
  * spec/data/network/gcompute_instance_group/success3~name.yaml
  * spec/data/network/gcompute_instance_group/success3~title.yaml
  * spec/data/network/gcompute_instance_group_manager/success1~name.yaml
  * spec/data/network/gcompute_instance_group_manager/success1~title.yaml
  * spec/data/network/gcompute_instance_group_manager/success2~name.yaml
  * spec/data/network/gcompute_instance_group_manager/success2~title.yaml
  * spec/data/network/gcompute_instance_group_manager/success3~name.yaml
  * spec/data/network/gcompute_instance_group_manager/success3~title.yaml
  * spec/data/network/gcompute_instance_template/success1~name.yaml
  * spec/data/network/gcompute_instance_template/success1~title.yaml
  * spec/data/network/gcompute_instance_template/success2~name.yaml
  * spec/data/network/gcompute_instance_template/success2~title.yaml
  * spec/data/network/gcompute_instance_template/success3~name.yaml
  * spec/data/network/gcompute_instance_template/success3~title.yaml
  * spec/data/network/gcompute_license/success1~name.yaml
  * spec/data/network/gcompute_license/success1~title.yaml
  * spec/data/network/gcompute_license/success2~name.yaml
  * spec/data/network/gcompute_license/success2~title.yaml
  * spec/data/network/gcompute_license/success3~name.yaml
  * spec/data/network/gcompute_license/success3~title.yaml
  * spec/data/network/gcompute_machine_type/success1~name.yaml
  * spec/data/network/gcompute_machine_type/success1~title.yaml
  * spec/data/network/gcompute_machine_type/success2~name.yaml
  * spec/data/network/gcompute_machine_type/success2~title.yaml
  * spec/data/network/gcompute_machine_type/success3~name.yaml
  * spec/data/network/gcompute_machine_type/success3~title.yaml
  * spec/data/network/gcompute_network/success1~name.yaml
  * spec/data/network/gcompute_network/success1~title.yaml
  * spec/data/network/gcompute_network/success2~name.yaml
  * spec/data/network/gcompute_network/success2~title.yaml
  * spec/data/network/gcompute_network/success3~name.yaml
  * spec/data/network/gcompute_network/success3~title.yaml
  * spec/data/network/gcompute_region/success1~name.yaml
  * spec/data/network/gcompute_region/success1~title.yaml
  * spec/data/network/gcompute_region/success2~name.yaml
  * spec/data/network/gcompute_region/success2~title.yaml
  * spec/data/network/gcompute_region/success3~name.yaml
  * spec/data/network/gcompute_region/success3~title.yaml
  * spec/data/network/gcompute_route/success1~name.yaml
  * spec/data/network/gcompute_route/success1~title.yaml
  * spec/data/network/gcompute_route/success2~name.yaml
  * spec/data/network/gcompute_route/success2~title.yaml
  * spec/data/network/gcompute_route/success3~name.yaml
  * spec/data/network/gcompute_route/success3~title.yaml
  * spec/data/network/gcompute_router/success1~name.yaml
  * spec/data/network/gcompute_router/success1~title.yaml
  * spec/data/network/gcompute_router/success2~name.yaml
  * spec/data/network/gcompute_router/success2~title.yaml
  * spec/data/network/gcompute_router/success3~name.yaml
  * spec/data/network/gcompute_router/success3~title.yaml
  * spec/data/network/gcompute_snapshot/success1~name.yaml
  * spec/data/network/gcompute_snapshot/success1~title.yaml
  * spec/data/network/gcompute_snapshot/success2~name.yaml
  * spec/data/network/gcompute_snapshot/success2~title.yaml
  * spec/data/network/gcompute_snapshot/success3~name.yaml
  * spec/data/network/gcompute_snapshot/success3~title.yaml
  * spec/data/network/gcompute_ssl_certificate/success1~name.yaml
  * spec/data/network/gcompute_ssl_certificate/success1~title.yaml
  * spec/data/network/gcompute_ssl_certificate/success2~name.yaml
  * spec/data/network/gcompute_ssl_certificate/success2~title.yaml
  * spec/data/network/gcompute_ssl_certificate/success3~name.yaml
  * spec/data/network/gcompute_ssl_certificate/success3~title.yaml
  * spec/data/network/gcompute_subnetwork/success1~name.yaml
  * spec/data/network/gcompute_subnetwork/success1~title.yaml
  * spec/data/network/gcompute_subnetwork/success2~name.yaml
  * spec/data/network/gcompute_subnetwork/success2~title.yaml
  * spec/data/network/gcompute_subnetwork/success3~name.yaml
  * spec/data/network/gcompute_subnetwork/success3~title.yaml
  * spec/data/network/gcompute_target_http_proxy/success1~name.yaml
  * spec/data/network/gcompute_target_http_proxy/success1~title.yaml
  * spec/data/network/gcompute_target_http_proxy/success2~name.yaml
  * spec/data/network/gcompute_target_http_proxy/success2~title.yaml
  * spec/data/network/gcompute_target_http_proxy/success3~name.yaml
  * spec/data/network/gcompute_target_http_proxy/success3~title.yaml
  * spec/data/network/gcompute_target_https_proxy/success1~name.yaml
  * spec/data/network/gcompute_target_https_proxy/success1~title.yaml
  * spec/data/network/gcompute_target_https_proxy/success2~name.yaml
  * spec/data/network/gcompute_target_https_proxy/success2~title.yaml
  * spec/data/network/gcompute_target_https_proxy/success3~name.yaml
  * spec/data/network/gcompute_target_https_proxy/success3~title.yaml
  * spec/data/network/gcompute_target_pool/success1~name.yaml
  * spec/data/network/gcompute_target_pool/success1~title.yaml
  * spec/data/network/gcompute_target_pool/success2~name.yaml
  * spec/data/network/gcompute_target_pool/success2~title.yaml
  * spec/data/network/gcompute_target_pool/success3~name.yaml
  * spec/data/network/gcompute_target_pool/success3~title.yaml
  * spec/data/network/gcompute_target_ssl_proxy/success1~name.yaml
  * spec/data/network/gcompute_target_ssl_proxy/success1~title.yaml
  * spec/data/network/gcompute_target_ssl_proxy/success2~name.yaml
  * spec/data/network/gcompute_target_ssl_proxy/success2~title.yaml
  * spec/data/network/gcompute_target_ssl_proxy/success3~name.yaml
  * spec/data/network/gcompute_target_ssl_proxy/success3~title.yaml
  * spec/data/network/gcompute_target_tcp_proxy/success1~name.yaml
  * spec/data/network/gcompute_target_tcp_proxy/success1~title.yaml
  * spec/data/network/gcompute_target_tcp_proxy/success2~name.yaml
  * spec/data/network/gcompute_target_tcp_proxy/success2~title.yaml
  * spec/data/network/gcompute_target_tcp_proxy/success3~name.yaml
  * spec/data/network/gcompute_target_tcp_proxy/success3~title.yaml
  * spec/data/network/gcompute_target_vpn_gateway/success1~name.yaml
  * spec/data/network/gcompute_target_vpn_gateway/success1~title.yaml
  * spec/data/network/gcompute_target_vpn_gateway/success2~name.yaml
  * spec/data/network/gcompute_target_vpn_gateway/success2~title.yaml
  * spec/data/network/gcompute_target_vpn_gateway/success3~name.yaml
  * spec/data/network/gcompute_target_vpn_gateway/success3~title.yaml
  * spec/data/network/gcompute_url_map/success1~name.yaml
  * spec/data/network/gcompute_url_map/success1~title.yaml
  * spec/data/network/gcompute_url_map/success2~name.yaml
  * spec/data/network/gcompute_url_map/success2~title.yaml
  * spec/data/network/gcompute_url_map/success3~name.yaml
  * spec/data/network/gcompute_url_map/success3~title.yaml
  * spec/data/network/gcompute_vpn_tunnel/success1~name.yaml
  * spec/data/network/gcompute_vpn_tunnel/success1~title.yaml
  * spec/data/network/gcompute_vpn_tunnel/success2~name.yaml
  * spec/data/network/gcompute_vpn_tunnel/success2~title.yaml
  * spec/data/network/gcompute_vpn_tunnel/success3~name.yaml
  * spec/data/network/gcompute_vpn_tunnel/success3~title.yaml
  * spec/data/network/gcompute_zone/success1~name.yaml
  * spec/data/network/gcompute_zone/success1~title.yaml
  * spec/data/network/gcompute_zone/success2~name.yaml
  * spec/data/network/gcompute_zone/success2~title.yaml
  * spec/data/network/gcompute_zone/success3~name.yaml
  * spec/data/network/gcompute_zone/success3~title.yaml
  * spec/disk_spec.rb
  * spec/disk_type_spec.rb
  * spec/fake_auth.rb
  * spec/fake_cred.rb
  * spec/firewall_spec.rb
  * spec/foodcritic_spec.rb
  * spec/forwarding_rule_spec.rb
  * spec/global_address_spec.rb
  * spec/global_forwarding_rule_spec.rb
  * spec/health_check_spec.rb
  * spec/http_health_check_spec.rb
  * spec/https_health_check_spec.rb
  * spec/image_spec.rb
  * spec/instance_group_manager_spec.rb
  * spec/instance_group_spec.rb
  * spec/instance_spec.rb
  * spec/instance_template_spec.rb
  * spec/license_spec.rb
  * spec/machine_type_spec.rb
  * spec/network_blocker.rb
  * spec/network_blocker_spec.rb
  * spec/network_delete_spec.rb
  * spec/network_get_spec.rb
  * spec/network_post_spec.rb
  * spec/network_put_spec.rb
  * spec/network_spec.rb
  * spec/region_spec.rb
  * spec/route_spec.rb
  * spec/router_spec.rb
  * spec/snapshot_spec.rb
  * spec/spec_helper.rb
  * spec/ssl_certificate_spec.rb
  * spec/subnetwork_spec.rb
  * spec/target_http_proxy_spec.rb
  * spec/target_https_proxy_spec.rb
  * spec/target_pool_spec.rb
  * spec/target_ssl_proxy_spec.rb
  * spec/target_tcp_proxy_spec.rb
  * spec/target_vpn_gateway_spec.rb
  * spec/test_constants.rb
  * spec/url_map_spec.rb
  * spec/vpn_tunnel_spec.rb
  * spec/zone_spec.rb

The list below contains all the files that were automatically sourced from a
central location:

  * Gemfile.lock
  * libraries/google/hash_utils.rb
  * libraries/google/string_utils.rb
  * LICENSE
  * spec/data/poor_recipe.rb
  * spec/hash_utils_spec.rb
  * spec/string_utils_spec.rb

[ruby-style-guide]: https://github.com/bbatsov/ruby-style-guide
[rubocop]: https://rubocop.readthedocs.io/en/latest/
